#! /usr/bin/python

import re
import signal
import subprocess
import sys

from AppKit import NSAlert
from AppKit import NSAlertFirstButtonReturn
from AppKit import NSAlertSecondButtonReturn
from AppKit import NSApp
from AppKit import NSImage


def run(cmd, input=None, shell=True, timeout=None):
    class TimeoutExpired(Exception): pass
    def run_timeout(signum, frame):
        if signum == signal.SIGALRM:
            raise TimeoutExpired()

    proc = subprocess.Popen(cmd, shell=shell, stdin=subprocess.PIPE,
        stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        if timeout:
            signal.signal(signal.SIGALRM, run_timeout)
            signal.alarm(timeout)
        stdout, stderr = proc.communicate(input)
        rc = proc.returncode
    except TimeoutExpired:
        proc.kill()
        rc, stdout, stderr = (-1, "", ("Command '%s' timed out after %s seconds" % (cmd, timeout)))
    finally:
        if timeout:
            signal.alarm(0)
    return rc, stdout, stderr


def check_update(label):
    exitCode, out, err = run("/usr/sbin/softwareupdate --list")
    if exitCode != 0:
        raise Exception("Failed to check for available software updates (code %d): %s" % (exitCode, err))
    
    updateFound = False
    for line in iter(out.splitlines()):
        if updateFound:
            match = re.match(r'^\s+Title: ([^,]+),', line)
            if not match:
                raise Exception("Failed to parse list of available software updates: %s" % out)
            title = match.group(1)
            restart = "restart," in line
            return (title, restart)
        if len(line) == 0:
            continue
        if line[0] == '*':
            if line.endswith(label):
                updateFound = True
                continue

    raise Exception("Software update '%s' is not available on the system" % label)


def download_update(label):
    exitCode, out, err = run("/usr/sbin/softwareupdate --download '%s' --no-scan" % label, timeout=2*60*60)
    if exitCode != 0:
        raise Exception("Failed to download the software update (code %d)" % exitCode)
    if "Downloaded:" not in out:
        raise Exception("Failed to download the software update: %s, %s" % (out, err))    
    return


def confirm_install(title, needRestart):
    messageText= "Update %s is ready to be installed" % title
    infoText = ""
    if needRestart:
        infoText = "Your Mac will automatically restart or shutdown if required to complete installation."

    alert = NSAlert.alloc().init()
    alert.setMessageText_(unicode(messageText, 'utf-8'))
    alert.setInformativeText_(unicode(infoText, 'utf-8'))
    alert.setIcon_(NSImage.alloc().initWithContentsOfFile_(
        u"/System/Library/CoreServices/Problem Reporter.app/Contents/Resources/ProblemReporter.icns"))

    alert.addButtonWithTitle_(u"Continue").setKeyEquivalent_(u"")
    alert.addButtonWithTitle_(u"Cancel").setKeyEquivalent_(u"")
    
    NSApp.activateIgnoringOtherApps_(True)
    result = alert.runModal()

    if result == NSAlertFirstButtonReturn:
        return

    raise Exception("Software update '%s' is cancelled by the user" % title)    


def install_update(label):
    exitCode, out, err = run("/usr/sbin/softwareupdate --install '%s' --no-scan --restart" % label)
    if exitCode != 0:
        raise Exception("Failed to install the software update (code %d)" % exitCode)
    if "Restarting" in out:
        print "Restarting..."
        return True
    if "Installed" not in out:
        raise Exception("Failed to install the software update: %s, %s" % (out, err))    
    return


try: 
    SU_LABEL = sys.argv[1]
    print "Checking update %s..." % SU_LABEL
    title, needRestart = check_update(SU_LABEL)
    print "Downloading..."
    download_update(SU_LABEL)
    print "Getting user consent..."
    confirm_install(title, needRestart)
    print "Installing..."
    install_update(SU_LABEL)
except Exception as e:
    print str(e)
    exit(1)